#!/usr/bin/env python
# -*- coding: utf-8 -*-
from bigbuild import get_base_url
from django.conf.urls import url, include
from bigbuild import views, feeds, sitemaps


basepatterns = [
    # Pages
    url(
        r'^$',
        views.PageListView.as_view(),
        name='bigbuild-page-list'
    ),
    url(
        r'^(?P<slug>[-\w]+)/$',
        views.PageDetailView.as_view(),
        name='bigbuild-page-detail'
    ),
    url(
        r'^(?P<slug>[-\w]+)/edit/$',
        views.PageUpdateView.as_view(),
        name='bigbuild-page-update'
    ),

    # Static assets
    url(
        r'^(?P<slug>[-\w]+)/static/(?P<path>.*)$',
        views.page_static_serve,
        name='bigbuild-page-detail-static'
    ),

    # Machine-readable feeds
    url(
        r'^sitemap.xml$',
        sitemaps.SitemapView.as_view(),
        name='bigbuild-sitemap'
    ),
    url(
        r'^google-news-sitemap.xml$',
        sitemaps.GoogleNewsSitemapView.as_view(),
        name='bigbuild-google-news-sitemap'
    ),
    url(
        r'^feeds/latest.xml$',
        feeds.LatestPages(),
        name="bigbuild-feeds-latest"
    ),
    url(
        r'^robots.txt$',
        views.RobotsView.as_view(),
        name="bigbuild-robots-txt"
    ),
]


urlpatterns = [
    # Index redirect
    url(
        r'^$',
        views.IndexRedirectView.as_view(),
        name="bigbuild-index-redirect"
    ),

    # All the base patterns prefixed
    url('^{}'.format(get_base_url().lstrip("/")), include(basepatterns)),
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import io
import os
import codecs
import shutil
import logging
from bigbuild.models import BasePage
from django.template import Engine, Context
from django.utils.encoding import python_2_unicode_compatible
from bigbuild.serializers import BigBuildFrontmatterSerializer
logger = logging.getLogger(__name__)


@python_2_unicode_compatible
class Page(BasePage):
    """
    A custom page published via static.latimes.com
    """
    def __str__(self):
        return self.slug

    def delete(self):
        """
        Delete the page directory.
        """
        shutil.rmtree(self.page_directory_path)

    @property
    def frontmatter_path(self):
        """
        Returns the metadata.md path where this page will be configured.
        """
        return os.path.join(self.page_directory_path, 'metadata.md')

    def create_directory(
        self,
        force=False,
        index_template_name='bigbuild/pages/default_index.html',
        index_template_context={},
    ):
        """
        Creates a new directory for the page.

        Returns the path to the directory that has been created, which is
        the same as the self.page_directory_path property.

        Throws an error if the directory already exists. You can force it
        to overwrite a pre-existing directory by submitting the force keyword
        argument as true.
        """
        # Check if the directory exists already and act accordingly
        if os.path.exists(self.page_directory_path):
            if force:
                shutil.rmtree(self.page_directory_path)
            else:
                raise ValueError(
                    "Page directory already exists at %s" % self.page_directory_path
                )

        # Make the directory
        os.mkdir(self.page_directory_path)

        # Add the index template
        self.write_index(
            template_name=index_template_name,
            template_context=index_template_context,
        )

        # Create the metadata configuration file
        self.write_frontmatter()

        # Create the ./static/ directory for extra stuff
        self.write_static()

        # Create the checklist file
        self.write_checklist()

        # Return the directory path
        return self.page_directory_path

    def write_index(self, template_name, template_context={}):
        """
        Creates index.html in the page directory.
        """
        template = self.get_template(template_name)
        html = template.render(Context(template_context))
        index_path = os.path.join(self.page_directory_path, 'index.html')
        with codecs.open(index_path, 'w', 'utf-8') as f:
            f.write(html)

    def write_frontmatter(self, path=None):
        """
        Creates metadata.yaml in the page directory, or a supplied target directory.
        """
        serializer = BigBuildFrontmatterSerializer()
        with io.open(path or self.frontmatter_path, 'w', encoding='utf8') as f:
            f.write(serializer.serialize([self]))

    def write_static(self):
        """
        Creates a ./static/ subdirectory within the page directory.
        """
        static_path = os.path.join(self.page_directory_path, 'static')
        os.mkdir(static_path)

        css_path = os.path.join(static_path, 'style.css')
        css = self.get_template("bigbuild/pages/style.css-tpl").source
        with codecs.open(css_path, 'w', 'utf-8') as f:
            f.write(css)

        js_path = os.path.join(static_path, 'app.js')
        js = self.get_template("bigbuild/pages/app.js-tpl").source
        with codecs.open(js_path, 'w', 'utf-8') as f:
            f.write(js)

    def write_checklist(self):
        """
        Creates checklist.md in the page directory.
        """
        template = self.get_template("bigbuild/pages/checklist.md-tpl")
        md = template.render(Context(dict(object=self)))
        checklist_path = os.path.join(self.page_directory_path, 'checklist.md')
        with codecs.open(checklist_path, 'w', 'utf-8') as f:
            f.write(md)

    def get_template(self, name):
        """
        Returns a Django template of the provided name ready to render.
        """
        engine = Engine.get_default()
        return engine.get_template(name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import six
import sys
import csv
import json
import yaml
import codecs
import logging
import archieml
import validictory
import frontmatter
from django.apps import apps
from datetime import datetime
from django.core.serializers.base import DeserializationError
from bigbuild.exceptions import MissingRecommendedMetadataWarning
from django.core.serializers.json import Serializer as JSONSerializer
from django.core.serializers.pyyaml import Serializer as YAMLSerializer
from django.core.serializers.python import Deserializer as PythonDeserializer
logger = logging.getLogger(__name__)


class BigBuildJSONSerializer(JSONSerializer):
    """
    A custom JSON serializer for bigbuild models.
    """
    pass


def BigBuildJSONDeserializer(stream_or_string, **options):
    """
    Deserialize a stream or string of JSON data.
    """
    if not isinstance(stream_or_string, (bytes, six.string_types)):
        stream_or_string = stream_or_string.read()
    if isinstance(stream_or_string, bytes):
        stream_or_string = stream_or_string.decode('utf-8')
    try:
        objects = json.loads(stream_or_string)
        for obj in PythonDeserializer(objects, **options):
            yield obj
    except GeneratorExit:
        raise
    except Exception as e:
        # Map to deserializer error
        six.reraise(DeserializationError, DeserializationError(e), sys.exc_info()[2])


class BigBuildFrontmatterSerializer(YAMLSerializer):
    """
    A custom YAML frontmatter serializer for bigbuild models.
    """
    def get_dump_object(self, obj):
        # Restructure to YAML schema preferred by frontmatter
        obj.metadata = self.get_metadata(obj)
        # Pass it out
        return obj

    def end_serialization(self):
        # Dump each object into the stream as as frontmatter
        content = [frontmatter.dumps(o) for o in self.objects]
        # Write out as text to the stream in a safe string format
        [self.stream.write(six.text_type(chunk)) for chunk in content]

    def get_metadata(self, obj):
        """
        Returns the structured "metadata" associated with this page expected by Jekyll's frontmatter.
        """
        # Structure the metadata
        d = dict([
            ('headline', obj.headline),
            ('byline', obj.byline),
            ('description', obj.description),
            ('image_url', obj.image_url),
            ('pub_date', obj.pub_date),
            ('published', obj.published),
            ('show_in_feeds', obj.show_in_feeds),
        ])
        if obj.extra:
            d['extra'] = obj.extra
        if obj.data:
            d['data'] = obj.data

        # Make sure the page is valid
        if not self.is_metadata_valid(d):
            raise ValueError("Metadata is not valid for %s" % obj)

        return d

    def is_metadata_valid(self, metadata):
        """
        Tests if the metadata is valid and returns True or False.
        """
        schema = {
            "type": "object",
            "properties": {
                'headline': {"type": "string", "blank": True},
                'byline': {"type": "string", "blank": True},
                'description': {"type": "string", "blank": True},
                'image_url': {"type": "any", "blank": True},
                'pub_date': {"type": "any"},
                'published': {"type": "boolean"},
                'show_in_feeds': {"type": "boolean"},
            }
        }
        try:
            validictory.validate(metadata, schema)
            if not isinstance(metadata['pub_date'], datetime):
                return False
            return True
        except:
            return False


class BaseBigBuildFrontmatterDeserializer(object):
    """
    Abstract method for deserializing YAML data from Jekyll's frontmatter format.
    """
    def deserialize(self, slug):
        """
        Retrieves the provided slug and returns a bigbuild object
        """
        logger.debug("Retrieving {} as {} object".format(
            slug,
            self.model.__name__
        ))
        obj = self.model.create(slug=slug, skip_create_directory=True)
        try:
            # Sync the metadata from the YAML frontmatter with the object
            obj = self.set_metadata(obj)
            # Pass it out
            return obj
        except Exception as e:
            # Map to deserializer error
            six.reraise(DeserializationError, DeserializationError(e), sys.exc_info()[2])

    def set_metadata(self, obj):
        """
        Syncs metadata with YAML frontmatter with provided bigbuild model object.
        """
        stream = open(obj.frontmatter_path, 'r')
        post = frontmatter.load(stream)

        # Set the basic frontmatter metadata
        obj.headline = post.metadata['headline']
        obj.byline = post.metadata['byline']
        obj.description = post.metadata['description']
        obj.image_url = post.metadata['image_url']
        obj.pub_date = post.metadata['pub_date']
        obj.published = post.metadata['published']
        obj.show_in_feeds = post.metadata['show_in_feeds']

        # Attach any extra stuff
        try:
            obj.extra = post.metadata['extra']
        except KeyError:
            obj.extra = {}

        try:
            obj.data = post.metadata['data']
        except KeyError:
            obj.data = {}

        # Pull in the content as is
        obj.content = post.content

        # Render it out as flat HTML
        obj.content = obj.rendered_content

        # Make sure the page has recommended metadata
        # ... if it's ready to publish
        if obj.pub_status in ['live', 'pending']:
            if not obj.has_recommended_metadata():
                logger.warn(MissingRecommendedMetadataWarning(obj))

        # Pass it back out
        return obj


class PageFrontmatterDeserializer(BaseBigBuildFrontmatterDeserializer):
    """
    Deserializes Jekyll frontmatter from YAML to a bigbuild Page object.
    """
    def __init__(self):
        self.model = apps.get_app_config('bigbuild').get_model('Page')

    def set_metadata(self, obj):
        """
        Extends the base deserialization with techniques for syncing data files.
        """
        obj = super(PageFrontmatterDeserializer, self).set_metadata(obj)

        # Reopen the YAML frontmatter
        stream = open(obj.frontmatter_path, 'r')
        post = frontmatter.load(stream)

        # Loop through any data files
        for key, path in post.metadata.get('data', {}).items():

            # Generate the path if it's stored in the default `data` directory
            data_dir = os.path.join(obj.page_directory_path, 'data')
            p = os.path.join(data_dir, path)
            # If it doesn't exist, see if it's in another folder
            if not os.path.exists(p):
                p = os.path.join(obj.page_directory_path, path)
                # If it's not there either, throw an error
                if not os.path.exists(p):
                    logging.debug("Data file could not be found at %s" % p)

            # Open the file
            with codecs.open(p, 'r') as f:
                # If it's a CSV file open it that way...
                if p.endswith(".csv"):
                    obj.data_objects[key] = list(csv.DictReader(f))
                # If it's a JSON file open it this way ...
                elif p.endswith(".json"):
                    obj.data_objects[key] = json.load(f)
                # If it's a YAML file open it t'other way ...
                elif (p.endswith(".yml") or p.endswith(".yaml")):
                    obj.data_objects[key] = yaml.load(f)
                elif p.endswith(".aml"):
                    obj.data_objects[key] = archieml.load(f)
                # If it's none of those throw an error.
                else:
                    logging.debug("Data file at %s not recognizable type" % path)

        # Pass it back out
        return obj


class ArchivedPageFrontmatterDeserializer(BaseBigBuildFrontmatterDeserializer):
    """
    Deserializes Jekyll frontmatter from YAML to a bigbuild ArchivedPage object.
    """
    def __init__(self):
        self.model = apps.get_app_config('bigbuild').get_model('ArchivedPage')


#
# Lookups
#


deserializers = {
    'Page': PageFrontmatterDeserializer,
    'ArchivedPage': ArchivedPageFrontmatterDeserializer
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import logging
import bigbuild
import jsonfield
from django.db import models
from datetime import datetime
from django.conf import settings
from django.utils import timezone
from greeking import latimes_ipsum
from django.test import RequestFactory
from bigbuild import context_processors
from bigbuild.serializers import deserializers
from django.template import Engine, RequestContext
from django.utils.encoding import python_2_unicode_compatible
logger = logging.getLogger(__name__)


@python_2_unicode_compatible
class BasePage(models.Model):
    """
    An abstract base class with methods shared by all page classes.
    """
    slug = models.SlugField(primary_key=True, max_length=500)
    headline = models.CharField(max_length=1000, blank=True, default="")
    byline = models.CharField(max_length=1000, blank=True, default="")
    description = models.TextField(blank=True)
    image_url = models.CharField(max_length=1000, blank=True, default="")
    pub_date = models.DateTimeField(default=datetime.now)
    published = models.BooleanField(default=False)
    show_in_feeds = models.BooleanField(default=True)
    content = models.TextField(blank=True)
    extra = jsonfield.JSONField(blank=True)
    data = jsonfield.JSONField(blank=True)

    class Meta:
        abstract = True

    def __str__(self):
        return self.slug

    #
    # URLs
    #

    @models.permalink
    def get_absolute_url(self):
        return ('bigbuild-page-detail', [self.slug])

    def get_static_url(self):
        return os.path.join(self.get_absolute_url(), 'static') + "/"

    #
    # Dynamic content
    #

    @property
    def rendered_content(self):
        """
        Returns the page's contents, which can contain Django templating tags, rendered as simple HTML.
        """
        engine = Engine.get_default()
        engine.dirs.extend([
            bigbuild.get_page_directory(),
            os.path.join(bigbuild.get_archive_directory(), 'static')
        ])
        template = engine.from_string(self.content)
        context = RequestContext(
            RequestFactory().get(self.get_absolute_url()),
            {
                "object": self,
                'STATIC_URL': self.get_static_url()
            },
            [context_processors.bigbuild]
        )
        return template.render(context)

    #
    # Actions
    #

    @classmethod
    def create(cls, **fields):
        """
        Create a new object.
        """
        # Pop out any extra inputs for the directory
        try:
            force = fields.pop("force")
        except KeyError:
            force = False

        try:
            index_template_name = fields.pop("index_template_name")
        except KeyError:
            index_template_name = 'bigbuild/pages/default_index.html'

        try:
            index_template_context = fields.pop("index_template_context")
        except KeyError:
            index_template_context = {}

        try:
            skip_create_directory = fields.pop("skip_create_directory")
        except KeyError:
            skip_create_directory = False

        # Create the object
        obj = cls(**fields)

        # Cache attribute to store rendered data files
        obj.data_objects = {}

        if not skip_create_directory:

            # If the directory already exists and we're not forcing creation
            # an error should be thrown.
            if os.path.exists(obj.page_directory_path) and not force:
                raise ValueError('Page directory already exists')
            elif os.path.exists(obj.archive_dynamic_directory_path) and not force:
                raise ValueError('Page directory already exists')

            # Create directory with any extra options passed in
            obj.create_directory(
                force=force,
                index_template_name=index_template_name,
                index_template_context=index_template_context,
            )

        # Return the object
        return obj

    def delete(self):
        """
        Delete the page directory.
        """
        raise NotImplementedError

    def build(self):
        """
        Builds this page via the PageDetailView.
        """
        from bigbuild.views import PageDetailView
        view = PageDetailView()
        view.build_object(self)

    def refresh_from_yaml(self):
        """
        Reads in the frontmatter from metadata.yaml and syncs it with the object.
        """
        deserializer = deserializer = deserializers[self.__class__.__name__]()
        yaml_obj = deserializer.deserialize(self.slug)
        for field in yaml_obj._meta.fields:
            setattr(self, field.name, getattr(yaml_obj, field.name))
        self.data_objects = yaml_obj.data_objects or {}

    #
    # File pathing
    #

    @property
    def page_directory_path(self):
        """
        Returns the directory path where this page will be stored.
        """
        return os.path.join(bigbuild.get_page_directory(), self.slug)

    @property
    def build_directory_path(self):
        """
        Returns the directory path where this page will be "baked" as flat files.
        """
        return os.path.join(bigbuild.get_build_directory(), self.get_absolute_url().lstrip("/"))

    @property
    def frontmatter_path(self):
        """
        Returns the metadata.md path where this page will be configured.
        """
        raise NotImplementedError

    @property
    def archive_dynamic_directory_path(self):
        """
        Returns the path where this page's dynamic content would be archived, if it were archived.
        """
        return os.path.join(bigbuild.get_archive_directory(), 'dynamic', self.slug)

    @property
    def archive_static_directory_path(self):
        """
        Returns the path where this page's dynamic static would be archived, if it were archived.
        """
        return os.path.join(bigbuild.get_archive_directory(), 'static', self.slug)

    #
    # Validation
    #

    def has_recommended_metadata(self):
        """
        Tests if the metadata has the fields we recommend are present
        for every page have been filled.

        Returns True or False.
        """
        recommended_fields = ['headline', 'description', 'byline', 'image_url']
        boilerplate = latimes_ipsum.get_story()
        for f in recommended_fields:
            # If it's an empty string cry foul
            if not getattr(self, f, ''):
                return False
            if f == 'image_url':
                if getattr(self, f) == boilerplate.image.url:
                    return False
            # Same thing if the value is from our boilerplate
            else:
                if getattr(self, f) == getattr(boilerplate, f):
                    return False
        return True

    #
    # Publication controls
    #

    @property
    def pub_status(self):
        """
        Returns the publication status as a string.

        There are three values:

            - live
            - working
            - pending
        """
        # Drop all the working ones right away
        if self.published is not True:
            return 'working'
        # If it is published but has a future date, that means its pending
        if timezone.make_aware(self.pub_date) > timezone.now():
            return 'pending'
        # Otherwise it should go live
        return 'live'

    def is_live(self):
        """
        Tests if the object should be published on the live production site.

        Returns True or False.
        """
        status_dict = {
            'live': True,
            'pending': False,
            'working': False
        }
        return status_dict[self.pub_status]

    def should_build(self):
        """
        Tests if the object should be published in the current environment.

        Returns True or False
        """
        # Get the publication status of the current environment
        status = getattr(settings, 'BIGBUILD_PAGE_PUBLICATION_STATUS', 'working')
        # If we are in 'working' mode everything gets built
        if status == 'working':
            return True
        # Otherwise default to the object's status
        return self.is_live()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Slimmed down subclasses of django-compressor internals that aim to work
without django.contrib.staticfiles installed.
"""
import os
from django.apps import apps
from bigbuild.models import PageList
from compressor_toolkit.precompilers import BaseCompiler
toolkit_config = apps.get_app_config('compressor_toolkit')


def get_all_static():
    """
    Returns the static directory path of all dynamic pages.
    """
    return [os.path.join(p.page_directory_path, 'static') for p in PageList().dynamic_pages]


class ES6Compiler(BaseCompiler):
    """
    django-compressor pre-compiler for ES6 files.
    """
    command = toolkit_config.ES6_COMPILER_CMD
    options = (
        ('browserify_bin', toolkit_config.BROWSERIFY_BIN),
        # Pull all the static file paths from dynamic pages plus the node_modules install directory
        ('paths', os.pathsep.join(get_all_static() + [toolkit_config.NODE_MODULES])),
        ('node_modules', toolkit_config.NODE_MODULES)
    )
    infile_ext = '.js'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .models import Page
from django.forms import ModelForm


class PageForm(ModelForm):
    """
    Form for editing pages.
    """
    class Meta:
        model = Page
        fields = [
            'headline',
            'byline',
            'description',
            'image_url',
            'pub_date',
            'published',
            'show_in_feeds',
            'content',
            'extra'
        ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import shutil
from django.core.management import call_command
from bigbuild.models import PageList, ArchivedPage
from django.core.management.base import BaseCommand, CommandError


class Command(BaseCommand):
    help = "Retrieve an archived page and return it to our dynamic page directory"
    args = "<slug>"

    def add_arguments(self, parser):
        parser.add_argument('slug', nargs='+', type=str)

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        # Loop through the slugs
        for slug in options['slug']:
            try:
                p = PageList()[slug]
            except:
                raise CommandError("Slug provided (%s) does not exist" % slug)

            # Make sure it's an archived page
            if not isinstance(p, ArchivedPage):
                raise CommandError("Slug (%s) is not a ArchivedPage object" % slug)

            # Make sure the dynamic page is archived (some of our older retired pages are static-only)
            if not os.path.exists(p.archive_dynamic_directory_path):
                raise CommandError("Slug (%s) does not have dynamic content archived. It cannot be retrieved." % slug)

            # Copy the archive's dynamic page to our live dynamic directory
            shutil.copytree(p.archive_dynamic_directory_path, p.page_directory_path)

            # Delete the archived directories
            p.delete()

        # Update the cache
        call_command("cachepages")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import six
import logging
import bigbuild
from django.conf import settings
from collections import Sequence
from bigbuild.serializers import deserializers
from bigbuild.serializers import BigBuildJSONDeserializer
logger = logging.getLogger(__name__)


class PageList(Sequence):
    """
    A list of all the Page and ArchivedPage objects in the application.
    """
    def __init__(self):
        # Set the page directories
        self.dynamic_directory = bigbuild.get_page_directory()
        self.archived_directory = bigbuild.get_archive_directory()

        # Set the cache path
        self.archived_cache_path = os.path.join(self.archived_directory, '.cache')

        # Pull the pages
        self.dynamic_pages = self.get_dynamic_pages()
        self.archived_pages = self.get_archived_pages()

        # Create a combined list of live pages and archived pages
        self.pages = []
        self.pages.extend(self.dynamic_pages)
        self.pages.extend(self.archived_pages)

        # Sort first by reverse chron
        self.pages = sorted(
            self.pages,
            key=lambda o: o.pub_date,
            reverse=True
        )

        # Then sort the working pages to the top
        self.pages = sorted(self.pages, key=lambda o: o.is_live())

    def __iter__(self):
        """
        What to use when this PageList is used in a for loop.
        """
        return iter(self.pages)

    def __len__(self):
        """
        Returns the number of pages in this PageList.
        """
        return len(self.pages)

    def __getitem__(self, key):
        """
        Pull items either with a slug key or an integer index.
        """
        if isinstance(key, six.string_types):
            try:
                return [i for i in self.pages if i.slug == key][0]
            except IndexError:
                raise KeyError("No page with that slug was found")
        elif isinstance(key, int):
            try:
                return self.pages[key]
            except IndexError:
                raise IndexError("No page with this key could be found")

    @staticmethod
    def get_page(slug, pagetype):
        """
        Returns a list of Page objects from the provided slug directory.
        """
        # Create an object from the directory slug
        deserializer = deserializers[pagetype]()
        return deserializer.deserialize(slug)

    def get_directory_list(self, path):
        """
        Returns the list of slugged page modules in the provided directory.
        """
        # Get the list of directories found on the filesystem
        dir_list = os.listdir(path)

        # Omit any that are on our blacklist
        blacklist = getattr(settings, 'BIGBUILD_PAGE_BLACKLIST', ['.DS_Store'])
        dir_list = [d for d in dir_list if d not in blacklist]

        # Return what remains
        return dir_list

    def get_dynamic_pages(self):
        """
        Returns a list of Page objects ready to be built in this environment.
        """
        dir_list = self.get_directory_list(self.dynamic_directory)
        deserializer = deserializers['Page']()
        page_list = [deserializer.deserialize(d) for d in dir_list]
        page_list = [p for p in page_list if p.should_build()]
        logger.debug("{} dynamic pages retrieved from YAML".format(len(page_list)))
        return page_list

    def get_archived_pages(self):
        """
        Returns a list of ArchivedPage objects ready to be built in this environment.
        """
        # Pull the cached data if it exists
        if os.path.exists(self.archived_cache_path):
            with open(self.archived_cache_path, 'r') as f:
                page_list = [
                    o.object for o in
                    BigBuildJSONDeserializer(f.read())
                    if o.object.should_build()
                ]
            logger.debug("{} archived pages retrieved from cache".format(len(page_list)))
            return page_list

        # Otherwise get them from the YAML
        static_archive_path = os.path.join(self.archived_directory, 'static')
        dir_list = self.get_directory_list(static_archive_path)
        deserializer = deserializers['ArchivedPage']()
        page_list = [deserializer.deserialize(d) for d in dir_list]
        page_list = [p for p in page_list if p.should_build()]
        logger.debug("{} archived pages retrieved from YAML".format(len(page_list)))
        return page_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import io
import os
import six
import bigbuild
from bigbuild.models import PageList
from django.core.management.base import BaseCommand
from bigbuild.serializers import BigBuildJSONSerializer


class Command(BaseCommand):
    help = "Cache page metadata to increase the application speed"

    def handle(self, *args, **options):
        # Set the cache path for archived pages
        archive_cache_path = os.path.join(bigbuild.get_archive_directory(), '.cache')

        # Delete it if it already exists
        if os.path.exists(archive_cache_path):
            os.remove(archive_cache_path)

        # Pull the live PageList from the YAML files
        page_list = PageList()

        # Create a JSON serializer
        serializer = BigBuildJSONSerializer()

        # Save the archived pages out to a new cache
        with io.open(archive_cache_path, 'w', encoding='utf8') as f:
            data = serializer.serialize(page_list.archived_pages)
            f.write(six.text_type(data))
